import numpy as np

def fx01(x):
    
    return np.exp(-1.*np.linalg.norm(x)**2)

def grad_fx01(x):
    
    return np.exp(-1.*np.linalg.norm(x)**2)*-2.*x

def fx02(x):
    
    return np.power(x, 2) * np.exp(-x) - x

def grad_fx02(x):
    
    return 2. * x * np.exp(-x) - np.power(x, 2) * np.exp(-x) - 1

def ArmijoGD(fx, grad_fx, x0, alpha=0.2, beta=0.8, epsilon=2e-4, max_iter=3000):
    
    if alpha < 0 or alpha > 1 or beta < 0 or beta > 1:
        print("Some of alpha or beta, out of range")
        raise ValueError
    
    step = 1
    x = x0
    if x==0.5 or x==2:
        print("x = {0}".format(x))
    
    while step < max_iter:
        beta_k = 1
        grad = grad_fx(x)
        
        while fx(x - beta_k*grad) > fx(x) - beta_k*alpha*(np.linalg.norm(grad)**2):
            beta_k = beta_k * beta 
        x = x - beta_k*grad
        
        if np.linalg.norm(grad) < epsilon:
            print("Almost Converged")
            break       
        step += 1
        print("{0} step -> x : {1}, f(x) : {2}".format(step, x, fx(x)))
        
    return x, fx(x)




ArmijoGD(fx01, grad_fx01, x)
ArmijoGD(fx02, grad_fx02, x0=0.5)
ArmijoGD(fx02, grad_fx02, x0=2)